@@ -1,9 +1,7 @@
 use std::fmt::{self, Display, Formatter};
 
-use itertools::Itertools;
-
 use crate::postgres::general::{Condition, OutputExpression, TableName, WithClause};
-use crate::tools::IntoIteratorOfSameType;
+use crate::tools::{joined, IntoIteratorOfSameType};
 
 /// Create a new `DELETE FROM` statement with the given table name.
 ///
@@ -116,12 +114,12 @@ impl Display for DeleteFrom {
 
         write!(f, "DELETE FROM {}", self.table_name,)?;
 
-        if self.where_.len() > 0 {
-            write!(f, " WHERE {}", self.where_.iter().join(" AND "))?;
+        if !self.where_.is_empty() {
+            write!(f, " WHERE {}", joined(&self.where_, " AND "))?;
         }
 
-        if self.returning.len() > 0 {
-            write!(f, " RETURNING {}", self.returning.iter().join(", "))?;
+        if !self.returning.is_empty() {
+            write!(f, " RETURNING {}", joined(&self.returning, ", "))?;
         }
 
         Ok(())
