@@ -2,10 +2,8 @@ mod values;
 
 use std::fmt::{self, Display, Formatter};
 
-use itertools::Itertools;
-
 use crate::postgres::general::{Column, Expression, OutputExpression, TableName, WithClause};
-use crate::tools::{IntoIteratorOfSameType, IntoNonZeroArray};
+use crate::tools::{joined, IntoIteratorOfSameType, IntoNonZeroArray};
 
 pub use values::{DefaultValues, Values, WithColumns, WithoutColumns};
 
@@ -46,15 +44,17 @@ pub use values::{DefaultValues, Values, WithColumns, WithoutColumns};
 /// ```
 ///
 /// ```
-/// use scooby::postgres::insert_into;
+/// use scooby::postgres::{insert_into, Parameters};
+///
+/// let mut params = Parameters::new();
 ///
 /// let sql = insert_into("Rectangle")
 ///     .columns(("width", "height"))
-///     .values([(10, 30)])
+///     .values([params.next_array()])
 ///     .returning("id")
 ///     .to_string();
 ///
-/// assert_eq!(sql, "INSERT INTO Rectangle (width, height) VALUES (10, 30) RETURNING id");
+/// assert_eq!(sql, "INSERT INTO Rectangle (width, height) VALUES ($1, $2) RETURNING id");
 /// ```
 pub fn insert_into(table_name: impl Into<TableName>) -> BareInsertInto {
     BareInsertInto {
@@ -103,9 +103,9 @@ impl BareInsertInto {
     /// ```
     /// use scooby::postgres::insert_into;
     ///
-    /// let sql = insert_into("Dummy").values([(1, 2), (3, 4)]).to_string();
+    /// let sql = insert_into("Dummy").values([("$1", "$2"), ("$3", "$4")]).to_string();
     ///
-    /// assert_eq!(sql, "INSERT INTO Dummy VALUES (1, 2), (3, 4)");
+    /// assert_eq!(sql, "INSERT INTO Dummy VALUES ($1, $2), ($3, $4)");
     /// ```
     pub fn values<T: IntoNonZeroArray<Expression, N>, const N: usize>(
         self,
@@ -130,10 +130,10 @@ impl BareInsertInto {
     ///
     /// let sql = insert_into("Dummy")
     ///     .columns(("col1", "col2"))
-    ///     .values([(1, 2), (3, 4)])
+    ///     .values([("$1", "$2"), ("$3", "$4")])
     ///     .to_string();
     ///
-    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES (1, 2), (3, 4)");
+    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES ($1, $2), ($3, $4)");
     pub fn columns<const N: usize>(
         self,
         columns: impl IntoNonZeroArray<Column, N>,
@@ -168,10 +168,10 @@ impl<const N: usize> InsertIntoColumnsBuilder<N> {
     ///
     /// let sql = insert_into("Dummy")
     ///     .columns(("col1", "col2"))
-    ///     .values([(1, 2), (3, 4)])
+    ///     .values([("$1", "$2"), ("$3", "$4")])
     ///     .to_string();
     ///
-    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES (1, 2), (3, 4)");
+    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES ($1, $2), ($3, $4)");
     pub fn values<T: IntoNonZeroArray<Expression, N>>(
         self,
         values: impl IntoIterator<Item = T>,
@@ -240,11 +240,11 @@ impl<const N: usize> InsertInto<WithColumns<N>> {
     ///
     /// let sql = insert_into("Dummy")
     ///     .columns(("col1", "col2"))
-    ///     .values([(1, 2)])
-    ///     .values([(3, 4), (5, 6)])
+    ///     .values([("$1", "$2")])
+    ///     .values([("$3", "$4"), ("$5", "$6")])
     ///     .to_string();
     ///
-    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES (1, 2), (3, 4), (5, 6)");
+    /// assert_eq!(sql, "INSERT INTO Dummy (col1, col2) VALUES ($1, $2), ($3, $4), ($5, $6)");
     pub fn values<T: IntoNonZeroArray<Expression, N>>(
         mut self,
         new_values: impl IntoIterator<Item = T>,
@@ -261,11 +261,11 @@ impl<const N: usize> InsertInto<WithoutColumns<N>> {
     /// use scooby::postgres::insert_into;
     ///
     /// let sql = insert_into("Dummy")
-    ///     .values([(1, 2)])
-    ///     .values([(3, 4), (5, 6)])
+    ///     .values([("$1", "$2")])
+    ///     .values([("$3", "$4"), ("$5", "$6")])
     ///     .to_string();
     ///
-    /// assert_eq!(sql, "INSERT INTO Dummy VALUES (1, 2), (3, 4), (5, 6)");
+    /// assert_eq!(sql, "INSERT INTO Dummy VALUES ($1, $2), ($3, $4), ($5, $6)");
     pub fn values<T: IntoNonZeroArray<Expression, N>>(
         mut self,
         new_values: impl IntoIterator<Item = T>,
@@ -283,8 +283,8 @@ impl<V: Values> Display for InsertInto<V> {
 
         write!(f, "INSERT INTO {} {}", self.table_name, self.values)?;
 
-        if self.returning.len() > 0 {
-            write!(f, " RETURNING {}", self.returning.iter().join(", "))?;
+        if !self.returning.is_empty() {
+            write!(f, " RETURNING {}", joined(&self.returning, ", "))?;
         }
 
         Ok(())
@@ -294,7 +294,7 @@ impl<V: Values> Display for InsertInto<V> {
 #[cfg(test)]
 mod tests {
     use crate::postgres::tools::tests::assert_correct_postgresql;
-    use crate::postgres::{insert_into, select, with};
+    use crate::postgres::{insert_into, select, with, Parameters};
 
     #[test]
     fn default_values() {
@@ -318,20 +318,12 @@ mod tests {
         assert_correct_postgresql(&sql, "INSERT INTO Dummy VALUES (a, b), (c, d), (e, f)");
     }
 
-    #[test]
-    fn no_columns_values_of_different_types() {
-        let sql = insert_into("Dummy")
-            .values([("\"Doug\"", 5, 1.76)])
-            .to_string();
-
-        assert_correct_postgresql(&sql, "INSERT INTO Dummy VALUES (\"Doug\", 5, 1.76)");
-    }
-
     // FIXME: This currently compiles and panics at runtime, but ideally should not even compile
     #[test]
     #[should_panic]
     fn zero_length_columns() {
-        let sql = insert_into("Dummy").columns([]).values([[]]).to_string();
+        let values: [[String; 0]; 1] = [[]];
+        let sql = insert_into("Dummy").columns([]).values(values).to_string();
 
         assert_correct_postgresql(&sql, "INSERT INTO Dummy () VALUES ");
     }
@@ -370,19 +362,6 @@ mod tests {
         );
     }
 
-    #[test]
-    fn value_various_types() {
-        let sql = insert_into("Dummy")
-            .columns(("name", "age", "height_in_meters"))
-            .values([("\"Doug\"", 5, 1.76)])
-            .to_string();
-
-        assert_correct_postgresql(
-            &sql,
-            "INSERT INTO Dummy (name, age, height_in_meters) VALUES (\"Doug\", 5, 1.76)",
-        );
-    }
-
     #[test]
     fn returning() {
         let sql = insert_into("Dummy")
@@ -421,4 +400,27 @@ mod tests {
             "WITH thing AS (SELECT 1 + 1) INSERT INTO Dummy VALUES (a)",
         );
     }
+
+    #[test]
+    fn array_params_with_columns() {
+        let mut params = Parameters::new();
+
+        let sql = insert_into("Dummy")
+            .columns(("col1", "col2"))
+            .values([params.next_array()])
+            .to_string();
+
+        assert_correct_postgresql(&sql, "INSERT INTO Dummy (col1, col2) VALUES ($1, $2)");
+    }
+
+    #[test]
+    fn array_params_without_columns() {
+        let mut params = Parameters::new();
+
+        let sql = insert_into("Dummy")
+            .values([params.next_array::<2>()])
+            .to_string();
+
+        assert_correct_postgresql(&sql, "INSERT INTO Dummy VALUES ($1, $2)");
+    }
 }
