@@ -6,10 +6,8 @@ mod order_by;
 use std::default::Default;
 use std::fmt::{self, Display, Formatter};
 
-use itertools::Itertools;
-
 use crate::postgres::general::{Condition, Expression, WithClause};
-use crate::tools::IntoIteratorOfSameType;
+use crate::tools::{joined, IntoIteratorOfSameType};
 
 pub use distinct::Distinct;
 pub use from_item::FromItem;
@@ -321,28 +319,28 @@ impl Display for Select {
             write!(f, " {}", distinct)?;
         }
 
-        if self.expressions.len() > 0 {
-            write!(f, " {}", self.expressions.iter().join(", "))?; // TODO: can be done without creating a temporary string?
+        if !self.expressions.is_empty() {
+            write!(f, " {}", joined(&self.expressions, ", "))?;
         }
 
-        if self.from.len() > 0 {
-            write!(f, " FROM {}", self.from.iter().join(", "))?;
+        if !self.from.is_empty() {
+            write!(f, " FROM {}", joined(&self.from, ", "))?;
         }
 
-        if self.where_.len() > 0 {
-            write!(f, " WHERE {}", self.where_.iter().join(" AND "))?;
+        if !self.where_.is_empty() {
+            write!(f, " WHERE {}", joined(&self.where_, " AND "))?;
         }
 
-        if self.group_by.len() > 0 {
-            write!(f, " GROUP BY {}", self.group_by.iter().join(", "))?;
+        if !self.group_by.is_empty() {
+            write!(f, " GROUP BY {}", joined(&self.group_by, ", "))?;
         }
 
-        if self.having.len() > 0 {
-            write!(f, " HAVING {}", self.having.iter().join(" AND "))?;
+        if !self.having.is_empty() {
+            write!(f, " HAVING {}", joined(&self.having, " AND "))?;
         }
 
-        if self.order_by.len() > 0 {
-            write!(f, " ORDER BY {}", self.order_by.iter().join(", "))?;
+        if !self.order_by.is_empty() {
+            write!(f, " ORDER BY {}", joined(&self.order_by, ", "))?;
         }
 
         if let Some(limit) = self.limit {
@@ -547,7 +545,6 @@ mod tests {
             .from(select("id").from("City").as_("x"))
             .to_string();
 
-        // FIXME: would be nice to guarantee that subselects MUST be aliased somehow?
         assert_correct_postgresql(&sql, "SELECT * FROM (SELECT id FROM City) AS x");
     }
 
@@ -639,7 +636,7 @@ mod tests {
     fn with_two_selects() {
         let sql = with("one")
             .as_(select("1 + 1"))
-            .and("two")
+            .and_with("two")
             .as_(select("2 + 2"))
             .select(("one.x", "two.x"))
             .from(("one", "two"))
@@ -659,7 +656,7 @@ mod tests {
                     .from("orders")
                     .group_by("region"),
             )
-            .and("top_regions")
+            .and_with("top_regions")
             .as_(select("region").from("regional_sales").where_(format!(
                 "total_sales > ({})",
                 select("SUM(total_sales)/10").from("regional_sales")
