@@ -1,11 +1,8 @@
 use std::fmt::{self, Display, Formatter};
 use std::iter::{once, Once};
 
-use itertools::Itertools;
-
 use crate::postgres::general::{Alias, TableName};
-use crate::postgres::queries::Select;
-use crate::tools::IntoIteratorOfSameType;
+use crate::tools::{joined, IntoIteratorOfSameType};
 
 use super::join::Join;
 
@@ -24,7 +21,7 @@ impl FromItem {
     }
 
     pub fn has_joins(&self) -> bool {
-        self.joins.len() > 0
+        !self.joins.is_empty()
     }
 
     pub fn add_join(&mut self, join: Join) {
@@ -36,8 +33,8 @@ impl Display for FromItem {
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
         write!(f, "{}", self.table_name)?;
 
-        if self.joins.len() > 0 {
-            write!(f, " {}", self.joins.iter().join(" "))? // ha, I'm joining joins, get it?
+        if self.has_joins() {
+            write!(f, " {}", joined(&self.joins, " "))?;
         }
 
         Ok(())
@@ -64,20 +61,6 @@ impl From<Alias> for FromItem {
     }
 }
 
-impl From<Select> for FromItem {
-    fn from(other: Select) -> FromItem {
-        FromItem::new(format!("({})", other))
-    }
-}
-
-impl IntoIteratorOfSameType<FromItem> for Select {
-    type Iterator = Once<FromItem>;
-
-    fn into_some_iter(self) -> Self::Iterator {
-        once(self.into())
-    }
-}
-
 impl<T> IntoIteratorOfSameType<T> for FromItem
 where
     T: From<FromItem>,
